#!/usr/bin/python
#
# Copyright 2014 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import azsec.util.common as util
import azsec.util.config as cfgutil
import os
import os.path
import string
import sys
import syslog
from azsec.util.common import waagent
from defusedxml.ElementTree import parse

# Initialize waagent
waagent.LoggerInit('/var/log/waagent.log','/dev/stdout')

# General Settings
scanConfigDir = util.scanConfigDir
baselineScript = os.path.dirname(os.path.realpath(__file__)) + "/apply-secbaseline.sh"

# Parse command line arguments, set options as appropriate
def ParseArguments():
    global scanConfigDir
    parser = argparse.ArgumentParser(description='Enable/disable security scanning options [' + scanConfigDir + ']')
    parser.add_argument(
        '-v', '--verbose', help='Write output to stdout', action='count')
    parser.add_argument(
        '-l', '--list', help='List scans along with their status (enabled/disabled)', action='store_true')
    parser.add_argument(
        '-n', '--name', help='Specify scanner by module name ("azsec.baselinescan.bscan" or "bscan")')
    parser.add_argument(
        '-c', '--cycle', help='Specify scanner cycle (or "disable")')
    parser.add_argument(
        '-d', '--dir', help='use the specified scanner configuration directory')
    parser.add_argument(
        '-ab', '--applybaseline', help='*Applies secure baseline settings to system*', action='store_true')
    args = parser.parse_args()
    # Set verbosity level for writing to console
    if args.verbose:
        util.runtimeVerbosity = args.verbose
    if args.dir:
        scanConfigDir=args.dir
    return args

def ApplyBaseline():
    print "**WARNING - This will script will make changes to your system! **"
    print "For additional details visit: http://aka.ms/jrt4vl"
    response = raw_input("Are you sure you want to proceed?[YES/NO]")
    if not (response == "YES"):
        util.WriteConsole(syslog.LOG_INFO,"Installation of secure baseline was cancelled by user input")
        return 0
    util.WriteConsole(syslog.LOG_INFO,"Initiating installation of secure baseline")
    error,msg = waagent.RunGetOutput(baselineScript)
    lines = string.split(msg, "\n")
    for line in lines:
        util.WriteLog(syslog.LOG_INFO, line)
    logpri = syslog.LOG_WARNING if error else syslog.LOG_INFO
    util.WriteConsole(logpri,"Installation of secure baseline is complete [{0}]".format(error))
    return 0

def ModifyConfigs(name, cycle, scanConfigMap):
    module = cfgutil.FindScannerName(name, scanConfigMap)
    if module is None:
        return 1
    scanConfigMap[module].cycle = cycle
    return cfgutil.ModifyConfigs(scanConfigMap.values(), scanConfigDir)

# Parse arguments and config files, report results
def main():
    args = ParseArguments()
    exitCode = 0
    if (args.applybaseline):
        exitCode += ApplyBaseline()
    # parse, verify, and modify (if valid) scanner configuration
    success, scanConfigMap = cfgutil.ParseConfig(scanConfigDir)
    if not success:
        util.WriteConsole(syslog.LOG_ERR, "Scanner configuration error(s) in "+scanConfigDir)
        exitCode += 1
    if args.name and args.cycle:
        exitCode += ModifyConfigs(args.name, args.cycle, scanConfigMap)
    elif args.name or args.cycle:
        util.WriteConsole(syslog.LOG_ERR, "Arguments -n and -c should be used in pair")
    if args.list:
        util.WriteDebug("Processing 'list' command")
        success, scanConfigMap = cfgutil.ParseConfig(scanConfigDir)
        for module in scanConfigMap.keys():
            print scanConfigMap[module].toString()
    sys.exit(exitCode)

main()
