#!/usr/bin/python
#
# Copyright 2015 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
import string
import syslog
import azsec.util.common as util
from azsec.util.common import waagent

securityList='/etc/apt/security.sources.list'

# Attempts to get a list of needed patches. Returns one of two values:
# True and an array of the needed patches
# False and an error message explaining the problem
def GetNeededPatches(distro):
    commands = {}
    commands[util.ubuntu] = 'grep "-security" /etc/apt/sources.list | sudo grep -v "#" > {0}; apt-get -qq -s upgrade -o Dir::Etc::SourceList={0}'.format(securityList)
    if not commands.has_key(distro):
        return False, distro + " is not a supported distro at this time"
    err,msg = waagent.RunGetOutput(commands[distro])
    if (err != 0):
        return False, "Error " + `err` + " encountered while executing command"
    lines = string.split(msg, "\n")
    regex = re.compile("^Inst")
    matchLines = filter(regex.match, lines)
    return True, matchLines
    
# Performs scan of needed patches and writes results to syslog
def RunScan(logFacility, args):
    syslog.openlog(facility=logFacility)
    print "Checking needed patches for " + util.distro
    success, result = GetNeededPatches(util.distro)
    if not success:
        util.WriteLog(syslog.LOG_ERR, result)
        syslog.closelog()
        return 1, 0
    if (len(result) < 1):
        util.WriteLog(syslog.LOG_INFO, "No Patches Needed")
    else:
        util.WriteLog(syslog.LOG_WARNING, `len(result)` + " Patch(es) Needed")
        for line in result:
            if (len(line) > 0):
                util.WriteLog(syslog.LOG_WARNING, "Patch needed: " + line)
    syslog.closelog()
    return 0, len(result)
