#!/usr/bin/python
#
# Copyright 2015 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import string
import syslog
import azsec.util.common as util
from azsec.util.common import waagent

# Performs scan of installed applications and writes results to syslog
def RunScan(logFacility, args):
    syslog.openlog(facility=logFacility)
    print "Gathering list of installed software"
    commands = {}
    commands[util.ubuntu] = 'dpkg -l | grep ^ii | awk \'{print $2,$3}\''
    if not commands.has_key(util.distro):
        util.WriteLog(syslog.LOG_ERR, util.distro + " is not supported at this time")
        syslog.closelog()
        return 1, 0
    error,msg = waagent.RunGetOutput(commands[util.distro])
    lines = string.split(msg, "\n")
    if (len(lines) < 1):
        util.WriteLog(syslog.LOG_ERROR, "No Applications installed")
        syslog.closelog()
        return 1, 0
    util.WriteLog(syslog.LOG_INFO, `len(lines)` + " Applications installed")
    for line in lines:
        if (len(line) > 0):
            util.WriteLog(syslog.LOG_INFO, "Installed Application: " + line)
    syslog.closelog()
    # No findings to report since this is just a list of installed software
    return 0, 0

# Performs after-scan remediation tasks. To be implemented.
def Remediate(logFacility, args):
    return 0
